require 'active_support/all'
require 'faraday'
require 'objspace'
require 'little-dutch/version'
require 'little-dutch/configuration'

module LittleDutch
  extend self

  CONFIG_FILE = File.expand_path File.join 'config', 'little-dutch.yml'

  def config
    @config ||= Configuration.new(CONFIG_FILE)
  end

  def run
    stop
    @runner = Thread.new { run_loop }
  end

  def start
    config.enabled ? run : :disabled
  end

  def stop
    @runner.try(:kill)
  end

  def profile_instance?
    url       = URI.parse(config.endpoint)
    url.path  = "/r/#{config.app_name}/#{config.instance_id}"
    url.query = { token: config.token }.to_param
    Faraday.get(url).status == 200
  rescue Faraday::ConnectionFailed
    false
  end

  private

  def run_loop
    @runcount = 0
    loop do
      if profile_instance?
        @runcount += 1
        post_json ObjectSpace.dump_all
      end
      sleep config.interval
    end
  end

  def connection
    @connection ||= Faraday.new config.endpoint do |conn|
      conn.request :multipart
      conn.request :url_encoded
      conn.adapter :net_http
    end
  end

  def post_json(file)
    url       = URI.parse(config.endpoint)
    url.path  = "/r/#{config.app_name}/#{config.instance_id}/#{@runcount}"
    url.query = { token: config.token }.to_param
    file_path = file.tap(&:close).path
    curl_cmd  = <<-SHELL.lines.map(&:strip).join(' ; ')
      curl -X POST --retry 10 -H "Transfer-Encoding: chunked" -H "Content-Type: application/text" --data-binary @#{file_path} #{url.to_s}
      rm -f #{file}
    SHELL
    Process.spawn curl_cmd, out: config.logfile, err: config.logfile, in: '/dev/null'
  end

end

require 'little-dutch/engine' if defined? Rails

